@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 
@@ -51,7 +53,7 @@
 
 BRASERO_PLUGIN_BOILERPLATE (BraseroLibburn, brasero_libburn, BRASERO_TYPE_JOB, BraseroJob);
 
-#define BRASERO_PVD_SIZE	32 * 2048
+#define BRASERO_PVD_SIZE	32ULL * 2048ULL
 
 struct _BraseroLibburnPrivate {
 	BraseroLibburnCtx *ctx;
@@ -59,6 +61,8 @@ struct _BraseroLibburnPrivate {
 	/* This buffer is used to capture Primary Volume Descriptor for
 	 * for overwrite media so as to "grow" the latter. */
 	unsigned char *pvd;
+
+	guint sig_handler:1;
 };
 typedef struct _BraseroLibburnPrivate BraseroLibburnPrivate;
 
@@ -187,7 +191,7 @@ brasero_libburn_src_read_xt (struct burn_source *src,
 
 static struct burn_source *
 brasero_libburn_create_fd_source (int fd,
-				  int size,
+				  gint64 size,
 				  unsigned char *pvd)
 {
 	struct burn_source *src;
@@ -223,7 +227,7 @@ brasero_libburn_add_track (struct burn_session *session,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("unable to set the source"));
+			     _("Libburn track could not be created"));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -231,7 +235,7 @@ brasero_libburn_add_track (struct burn_session *session,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("unable to add the track to the session"));
+			     _("Libburn track could not be created"));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -277,7 +281,8 @@ brasero_libburn_add_file_track (struct burn_session *session,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     strerror (errno));
+			     "%s",
+			     g_strerror (errno));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -290,7 +295,7 @@ brasero_libburn_setup_session_fd (BraseroLibburn *self,
 			          GError **error)
 {
 	int fd;
-	gint64 size;
+	gint64 size = 0;
 	BraseroTrackType type;
 	BraseroLibburnPrivate *priv;
 	BraseroBurnResult result = BRASERO_BURN_OK;
@@ -306,6 +311,7 @@ brasero_libburn_setup_session_fd (BraseroLibburn *self,
 		{
 			gint mode;
 
+			/* FIXME: implement other IMAGE types */
 			if (type.subtype.img_format == BRASERO_IMAGE_FORMAT_BIN)
 				mode = BURN_MODE1;
 			else
@@ -314,6 +320,7 @@ brasero_libburn_setup_session_fd (BraseroLibburn *self,
 			brasero_job_get_session_output_size (BRASERO_JOB (self),
 							     NULL,
 							     &size);
+
 			result = brasero_libburn_add_fd_track (session,
 							       fd,
 							       mode,
@@ -341,7 +348,7 @@ brasero_libburn_setup_session_fd (BraseroLibburn *self,
 								       dup (fd),
 								       BURN_AUDIO,
 								       size,
-								       priv->pvd,
+								       NULL,
 								       error);
 				if (result != BRASERO_BURN_OK)
 					return result;
@@ -378,19 +385,24 @@ brasero_libburn_setup_session_file (BraseroLibburn *self,
 		brasero_track_get_type (track, &type);
 		if (type.type == BRASERO_TRACK_TYPE_AUDIO) {
 			gchar *audiopath;
+			gint64 size;
 
 			audiopath = brasero_track_get_audio_source (track, FALSE);
+			brasero_track_get_audio_length (track, &size);
+			size = BRASERO_DURATION_TO_BYTES (size);
+
 			result = brasero_libburn_add_file_track (session,
 								 audiopath,
 								 BURN_AUDIO,
-								 -1,
-								 priv->pvd,
+								 size,
+								 NULL,
 								 error);
 			if (result != BRASERO_BURN_OK)
 				break;
 		}
 		else if (type.type == BRASERO_TRACK_TYPE_IMAGE) {
 			gchar *imagepath;
+			gint64 size;
 			gint mode;
 
 			if (type.subtype.img_format == BRASERO_IMAGE_FORMAT_BIN) {
@@ -407,12 +419,21 @@ brasero_libburn_setup_session_file (BraseroLibburn *self,
 			if (!imagepath)
 				return BRASERO_BURN_ERR;
 
+			result = brasero_track_get_image_size (track,
+							       NULL,
+							       NULL,
+							       &size,
+							       error);
+			if (result != BRASERO_BURN_OK)
+				return BRASERO_BURN_ERR;
+
 			result = brasero_libburn_add_file_track (session,
 								 imagepath,
 								 mode,
-								 -1,
+								 size,
 								 priv->pvd,
 								 error);
+			g_free (imagepath);
 		}
 		else
 			BRASERO_JOB_NOT_SUPPORTED (self);
@@ -457,11 +478,13 @@ brasero_libburn_start_record (BraseroLibburn *self,
 			      GError **error)
 {
 	guint64 rate;
+	gint64 blocks = 0;
 	BraseroMedia media;
 	BraseroBurnFlag flags;
 	BraseroBurnResult result;
 	BraseroLibburnPrivate *priv;
 	struct burn_write_opts *opts;
+	gchar reason [BURN_REASONS_LEN];
 
 	priv = BRASERO_LIBBURN_PRIVATE (self);
 
@@ -470,9 +493,7 @@ brasero_libburn_start_record (BraseroLibburn *self,
 	brasero_job_get_media (BRASERO_JOB (self), &media);
 
 	if (flags & (BRASERO_BURN_FLAG_MERGE|BRASERO_BURN_FLAG_APPEND)
-	&& (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS)
-	||  BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_RESTRICTED)
-	||  BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS_DL))
+	&&  BRASERO_MEDIUM_RANDOM_WRITABLE (media)
 	&& (media & BRASERO_MEDIUM_HAS_DATA))
 		priv->pvd = g_new0 (unsigned char, BRASERO_PVD_SIZE);
 
@@ -487,10 +508,7 @@ brasero_libburn_start_record (BraseroLibburn *self,
 	opts = burn_write_opts_new (priv->ctx->drive);
 	burn_write_opts_set_perform_opc (opts, 0);
 
-	if (!BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS)
-	&&  !BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_RESTRICTED)
-	&&  !BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS_DL)
-	&&  (flags & BRASERO_BURN_FLAG_DAO))
+	if (flags & BRASERO_BURN_FLAG_DAO)
 		burn_write_opts_set_write_type (opts,
 						BURN_WRITE_SAO,
 						BURN_BLOCK_SAO);
@@ -499,8 +517,11 @@ brasero_libburn_start_record (BraseroLibburn *self,
 						BURN_WRITE_TAO,
 						BURN_BLOCK_MODE1);
 
-		/* we also set the start block to write from if MERGE is set */
-		if (flags & BRASERO_BURN_FLAG_MERGE) {
+		/* we also set the start block to write from if MERGE is set.
+		 * That only for random writable media; for other media libburn
+		 * handles all by himself where to start writing. */
+		if (BRASERO_MEDIUM_RANDOM_WRITABLE (media)
+		&& (flags & BRASERO_BURN_FLAG_MERGE)) {
 			gint64 address = 0;
 
 			brasero_job_get_next_writable_address (BRASERO_JOB (self), &address);
@@ -510,12 +531,36 @@ brasero_libburn_start_record (BraseroLibburn *self,
 		}
 	}
 
+	if (!BRASERO_MEDIUM_RANDOM_WRITABLE (media))
+		burn_write_opts_set_multi (opts, (flags & BRASERO_BURN_FLAG_MULTI) != 0);
+
 	burn_write_opts_set_underrun_proof (opts, (flags & BRASERO_BURN_FLAG_BURNPROOF) != 0);
-	burn_write_opts_set_multi (opts, (flags & BRASERO_BURN_FLAG_MULTI) != 0);
 	burn_write_opts_set_simulate (opts, (flags & BRASERO_BURN_FLAG_DUMMY) != 0);
 
 	brasero_job_get_rate (BRASERO_JOB (self), &rate);
 	burn_drive_set_speed (priv->ctx->drive, rate, 0);
+
+	if (burn_precheck_write (opts, priv->ctx->disc, reason, 0) < 1) {
+		g_set_error (error,
+			     BRASERO_BURN_ERROR,
+			     BRASERO_BURN_ERROR_GENERAL,
+			     reason);
+		return BRASERO_BURN_ERR;
+	}
+
+	/* If we're writing to a disc remember that the session can't be under
+	 * 300 sectors (= 614400 bytes) */
+	brasero_job_get_session_output_size (BRASERO_JOB (self), &blocks, NULL);
+	if (blocks < 300)
+		brasero_job_set_output_size_for_current_track (BRASERO_JOB (self),
+							       300L - blocks,
+							       614400L - blocks * 2048);
+
+	if (!priv->sig_handler) {
+		burn_set_signal_handling ("brasero", NULL, 0);
+		priv->sig_handler = 1;
+	}
+
 	burn_disc_write (opts, priv->ctx->disc);
 	burn_write_opts_free (opts);
 
@@ -537,11 +582,11 @@ brasero_libburn_start_erase (BraseroLibburn *self,
 	int fd;
 
 	priv = BRASERO_LIBBURN_PRIVATE (self);
-	if (burn_disc_get_profile (priv->ctx->drive, &profile, prof_name) < 0) {
+	if (burn_disc_get_profile (priv->ctx->drive, &profile, prof_name) <= 0) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_GENERAL,
-			     _("no profile available for the medium"));
+			     BRASERO_BURN_ERROR_MEDIUM_INVALID,
+			     _("The disc is not supported"));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -552,6 +597,11 @@ brasero_libburn_start_erase (BraseroLibburn *self,
 	if (profile == BRASERO_SCSI_PROF_DVD_RW_RESTRICTED) {
 		if (!(flags & BRASERO_BURN_FLAG_FAST_BLANK)) {
 			/* leave libburn choose the best format */
+			if (!priv->sig_handler) {
+				burn_set_signal_handling ("brasero", NULL, 0);
+				priv->sig_handler = 1;
+			}
+
 			burn_disc_format (priv->ctx->drive,
 					  (off_t) 0,
 					  (1 << 4));
@@ -564,8 +614,12 @@ brasero_libburn_start_erase (BraseroLibburn *self,
 			 * Bit 4 is enforce (re)-format if needed
 			 * 0x26 is DVD+RW format is to be set from bit 8
 			 * in the latter case bit 7 needs to be set as 
-			 * well.
-			 */
+			 * well. */
+			if (!priv->sig_handler) {
+				burn_set_signal_handling ("brasero", NULL, 0);
+				priv->sig_handler = 1;
+			}
+
 			burn_disc_format (priv->ctx->drive,
 					  (off_t) 0,
 					  (1 << 2)|(1 << 4));
@@ -574,6 +628,11 @@ brasero_libburn_start_erase (BraseroLibburn *self,
 	}
 	else if (burn_disc_erasable (priv->ctx->drive)) {
 		/* This is mainly for CDRW and sequential DVD-RW */
+		if (!priv->sig_handler) {
+			burn_set_signal_handling ("brasero", NULL, 0);
+			priv->sig_handler = 1;
+		}
+
 		burn_disc_erase (priv->ctx->drive, (flags & BRASERO_BURN_FLAG_FAST_BLANK) != 0);
 		return BRASERO_BURN_OK;
 	}
@@ -584,10 +643,16 @@ brasero_libburn_start_erase (BraseroLibburn *self,
 	 * /dev/null */
 	fd = open ("/dev/null", O_RDONLY);
 	if (fd == -1) {
+		int errnum = errno;
+
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("/dev/null can't be opened"));
+			     /* Translators: first %s is the filename, second %s is the error
+			      * generated from errno */
+			     _("\"%s\" could not be opened (%s)"),
+			     "/dev/null",
+			     g_strerror (errnum));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -618,13 +683,21 @@ brasero_libburn_start_erase (BraseroLibburn *self,
 
 	if (burn_precheck_write (opts, priv->ctx->disc, reasons, 0) <= 0) {
 		burn_write_opts_free (opts);
+
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("libburn can't burn: %s"), reasons);
+			     /* Translators: %s is the error returned by libburn */
+			     _("An internal error occured (%s)"),
+			     reasons);
 		return BRASERO_BURN_ERR;
 	}
 
+	if (!priv->sig_handler) {
+		burn_set_signal_handling ("brasero", NULL, 0);
+		priv->sig_handler = 1;
+	}
+
 	burn_disc_write (opts, priv->ctx->disc);
 	burn_write_opts_free (opts);
 
@@ -688,6 +761,11 @@ brasero_libburn_stop (BraseroJob *job,
 	self = BRASERO_LIBBURN (job);
 	priv = BRASERO_LIBBURN_PRIVATE (self);
 
+	if (priv->sig_handler) {
+		priv->sig_handler = 0;
+		burn_set_signal_handling (NULL, NULL, 1);
+	}
+
 	if (priv->ctx) {
 		brasero_libburn_common_ctx_free (priv->ctx);
 		priv->ctx = NULL;
@@ -719,10 +797,12 @@ brasero_libburn_clock_tick (BraseroJob *job)
 
 	/* Double check that everything went well */
 	if (!burn_drive_wrote_well (priv->ctx->drive)) {
+		BRASERO_JOB_LOG (job, "Something went wrong");
 		brasero_job_error (job,
 				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_GENERAL,
-						_("an unknown error occured")));
+						BRASERO_BURN_ERROR_WRITE_MEDIUM,
+						_("An error occured while writing to disc")));
+		return BRASERO_BURN_OK;
 	}
 
 	/* That's finished */
@@ -748,10 +828,11 @@ brasero_libburn_clock_tick (BraseroJob *job)
 					BRASERO_PVD_SIZE,
 					0);
 	if (ret != 1) {
+		BRASERO_JOB_LOG (job, "Random write failed");
 		brasero_job_error (job,
 				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_GENERAL,
-						_("an unknown error occured")));
+						BRASERO_BURN_ERROR_WRITE_MEDIUM,
+						_("An error occured while writing to disc")));
 		return BRASERO_BURN_OK;
 	}
 
@@ -797,10 +878,6 @@ brasero_libburn_finalize (GObject *object)
 		priv->ctx = NULL;
 	}
 
-	/* Since the library is not needed any more call burn_finish ().
-	 * NOTE: it itself calls burn_abort (). */
-	burn_finish ();
-
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -828,7 +905,7 @@ brasero_libburn_export_caps (BraseroPlugin *plugin, gchar **error)
 					  BRASERO_MEDIUM_HAS_DATA|
 					  BRASERO_MEDIUM_BLANK;
 	const BraseroMedia media_dvd_rw_plus = BRASERO_MEDIUM_DVD|
-					       BRASERO_MEDIUM_DVD_DL|
+					       BRASERO_MEDIUM_DUAL_L|
 					       BRASERO_MEDIUM_PLUS|
 					       BRASERO_MEDIUM_RESTRICTED|
 					       BRASERO_MEDIUM_REWRITABLE|
@@ -842,43 +919,21 @@ brasero_libburn_export_caps (BraseroPlugin *plugin, gchar **error)
 
 	brasero_plugin_define (plugin,
 			       "libburn",
-			       _("libburn burns CD(RW), DVD+/-(RW)"),
+			       _("Libburn burns CD(RW), DVD+/-(RW)"),
 			       "Philippe Rouquier",
 			       15);
 
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_CD|
-				  BRASERO_MEDIUM_REWRITABLE|
-				  BRASERO_MEDIUM_WRITABLE|
-				  BRASERO_MEDIUM_BLANK,
-				  BRASERO_BURN_FLAG_DAO|
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_DUMMY|
-				  BRASERO_BURN_FLAG_NOGRACE,
-				  BRASERO_BURN_FLAG_NONE);
+	/* libburn has no OVERBURN capabilities */
 
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_CD|
-				  BRASERO_MEDIUM_REWRITABLE|
-				  BRASERO_MEDIUM_WRITABLE|
-				  BRASERO_MEDIUM_APPENDABLE|
-				  BRASERO_MEDIUM_HAS_AUDIO|
-				  BRASERO_MEDIUM_HAS_DATA,
-				  BRASERO_BURN_FLAG_APPEND|
-				  BRASERO_BURN_FLAG_MERGE|
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_DUMMY|
-				  BRASERO_BURN_FLAG_NOGRACE,
-				  BRASERO_BURN_FLAG_NONE);
+	/* CD(R)W */
+	BRASERO_PLUGIN_ADD_STANDARD_CDR_FLAGS (plugin, BRASERO_BURN_FLAG_OVERBURN);
+	BRASERO_PLUGIN_ADD_STANDARD_CDRW_FLAGS (plugin, BRASERO_BURN_FLAG_OVERBURN);
 
 	/* audio support for CDs only */
 	input = brasero_caps_audio_new (BRASERO_PLUGIN_IO_ACCEPT_PIPE|
 					BRASERO_PLUGIN_IO_ACCEPT_FILE,
-					BRASERO_AUDIO_FORMAT_RAW);
+					BRASERO_AUDIO_FORMAT_RAW|
+					BRASERO_AUDIO_FORMAT_44100);
 	
 	output = brasero_caps_disc_new (media_cd);
 	brasero_plugin_link_caps (plugin, output, input);
@@ -892,205 +947,29 @@ brasero_libburn_export_caps (BraseroPlugin *plugin, gchar **error)
 	brasero_plugin_link_caps (plugin, output, input);
 	g_slist_free (output);
 
-	/* ... and DVDs-R ... */
-	/* NOTE: DAO and multi are exclusive */
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVDR|
-				  BRASERO_MEDIUM_BLANK,
-				  BRASERO_BURN_FLAG_DAO|
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_DUMMY|
-				  BRASERO_BURN_FLAG_NOGRACE,
-				  BRASERO_BURN_FLAG_NONE);
-
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVDR|
-				  BRASERO_MEDIUM_BLANK,
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_DUMMY|
-				  BRASERO_BURN_FLAG_NOGRACE,
-				  BRASERO_BURN_FLAG_NONE);
-
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVDR|
-				  BRASERO_MEDIUM_APPENDABLE|
-				  BRASERO_MEDIUM_HAS_DATA,
-				  BRASERO_BURN_FLAG_APPEND|
-				  BRASERO_BURN_FLAG_MERGE|
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_DUMMY|
-				  BRASERO_BURN_FLAG_NOGRACE,
-				  BRASERO_BURN_FLAG_NONE);
-
-	/* NOTE: DVD+R don't have a dummy mode */
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVDR_PLUS|
-				  BRASERO_MEDIUM_BLANK,
-				  BRASERO_BURN_FLAG_DAO|
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_NOGRACE,
-				  BRASERO_BURN_FLAG_NONE);
-
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVDR_PLUS|
-				  BRASERO_MEDIUM_BLANK,
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_NOGRACE,
-				  BRASERO_BURN_FLAG_NONE);
-
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVDR_PLUS|
-				  BRASERO_MEDIUM_APPENDABLE|
-				  BRASERO_MEDIUM_HAS_DATA,
-				  BRASERO_BURN_FLAG_MERGE|
-				  BRASERO_BURN_FLAG_APPEND|
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_NOGRACE,
-				  BRASERO_BURN_FLAG_NONE);
-
+	/* ... and DVD-R and DVD+R ... */
 	output = brasero_caps_disc_new (media_dvd_w);
 	brasero_plugin_link_caps (plugin, output, input);
 	g_slist_free (output);
 
-	/* ... and DVDs-RW (sequential) */
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVDRW|
-				  BRASERO_MEDIUM_UNFORMATTED|
-				  BRASERO_MEDIUM_BLANK,
-				  BRASERO_BURN_FLAG_DAO|
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_DUMMY|
-				  BRASERO_BURN_FLAG_NOGRACE,
-				  BRASERO_BURN_FLAG_NONE);
-
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVDRW|
-				  BRASERO_MEDIUM_BLANK,
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_DUMMY|
-				  BRASERO_BURN_FLAG_NOGRACE,
-				  BRASERO_BURN_FLAG_NONE);
-
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVDRW|
-				  BRASERO_MEDIUM_APPENDABLE|
-				  BRASERO_MEDIUM_HAS_DATA,
-				  BRASERO_BURN_FLAG_MERGE|
-				  BRASERO_BURN_FLAG_APPEND|
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_DUMMY|
-				  BRASERO_BURN_FLAG_NOGRACE,
-				  BRASERO_BURN_FLAG_NONE);
+	BRASERO_PLUGIN_ADD_STANDARD_DVDR_PLUS_FLAGS (plugin, BRASERO_BURN_FLAG_OVERBURN);
+	BRASERO_PLUGIN_ADD_STANDARD_DVDR_FLAGS (plugin, BRASERO_BURN_FLAG_OVERBURN);
 
+	/* ... and DVD-RW (sequential) */
 	output = brasero_caps_disc_new (media_dvd_rw);
 	brasero_plugin_link_caps (plugin, output, input);
 	g_slist_free (output);
 
-	/* for DVD+/- restricted */
+	BRASERO_PLUGIN_ADD_STANDARD_DVDRW_FLAGS (plugin, BRASERO_BURN_FLAG_OVERBURN);
+
+	/* for DVD+/-RW restricted */
 	output = brasero_caps_disc_new (media_dvd_rw_plus);
 	brasero_plugin_link_caps (plugin, output, input);
 	g_slist_free (output);
 	g_slist_free (input);
 
-	/* for DVD-RW restricted overwrite */
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVD|
-				  BRASERO_MEDIUM_RESTRICTED|
-				  BRASERO_MEDIUM_REWRITABLE|
-				  BRASERO_MEDIUM_UNFORMATTED|
-				  BRASERO_MEDIUM_BLANK,
-				  BRASERO_BURN_FLAG_DAO|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_DUMMY|
-				  BRASERO_BURN_FLAG_NOGRACE|
-				  BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE,
-				  BRASERO_BURN_FLAG_MULTI);
-
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVD|
-				  BRASERO_MEDIUM_RESTRICTED|
-				  BRASERO_MEDIUM_REWRITABLE|
-				  BRASERO_MEDIUM_APPENDABLE|
-				  BRASERO_MEDIUM_CLOSED|
-				  BRASERO_MEDIUM_HAS_DATA,
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_DUMMY|
-				  BRASERO_BURN_FLAG_NOGRACE|
-				  BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE,
-				  BRASERO_BURN_FLAG_MULTI);
-
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVD|
-				  BRASERO_MEDIUM_RESTRICTED|
-				  BRASERO_MEDIUM_REWRITABLE|
-				  BRASERO_MEDIUM_APPENDABLE|
-				  BRASERO_MEDIUM_CLOSED|
-				  BRASERO_MEDIUM_HAS_DATA,
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_DUMMY|
-				  BRASERO_BURN_FLAG_NOGRACE|
-				  BRASERO_BURN_FLAG_MERGE,
-				  BRASERO_BURN_FLAG_MULTI);
-
-	/* for DVD+RW */
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVDRW_PLUS|
-				  BRASERO_MEDIUM_DVD_DL|
-				  BRASERO_MEDIUM_UNFORMATTED|
-				  BRASERO_MEDIUM_BLANK,
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_DAO|
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_NOGRACE,
-				  BRASERO_BURN_FLAG_MULTI);
-
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVDRW_PLUS|
-				  BRASERO_MEDIUM_DVD_DL|
-				  BRASERO_MEDIUM_APPENDABLE|
-				  BRASERO_MEDIUM_CLOSED|
-				  BRASERO_MEDIUM_HAS_DATA,
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_NOGRACE|
-				  BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE,
-				  BRASERO_BURN_FLAG_MULTI);
-
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVDRW_PLUS|
-				  BRASERO_MEDIUM_DVD_DL|
-				  BRASERO_MEDIUM_APPENDABLE|
-				  BRASERO_MEDIUM_CLOSED|
-				  BRASERO_MEDIUM_HAS_DATA,
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_NOGRACE|
-				  BRASERO_BURN_FLAG_MERGE,
-				  BRASERO_BURN_FLAG_MULTI);
+	BRASERO_PLUGIN_ADD_STANDARD_DVDRW_RESTRICTED_FLAGS (plugin, BRASERO_BURN_FLAG_OVERBURN);
+	BRASERO_PLUGIN_ADD_STANDARD_DVDRW_PLUS_FLAGS (plugin, BRASERO_BURN_FLAG_OVERBURN);
 
 	/* add blank caps */
 	output = brasero_caps_disc_new (BRASERO_MEDIUM_CD|
